@@ -8,7 +8,7 @@ use pyo3::prelude::*;
 /// # Returns
 /// The converted URL
 pub fn vcs_git_url_to_bzr_url(url: &str) -> url::Url {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let m = py.import("breezy.plugins.debian.directory").unwrap();
         m.call_method1("vcs_git_url_to_bzr_url", (url,))
             .unwrap()
