@@ -112,7 +112,7 @@ pub use workspace::reset_tree;
 ///
 /// This function imports the breezy.git module to ensure Git functionality is available.
 pub fn init_git() {
-    pyo3::Python::with_gil(|py| {
+    pyo3::Python::attach(|py| {
         py.import("breezy.git").unwrap();
     })
 }
@@ -121,12 +121,13 @@ pub fn init_git() {
 ///
 /// This function imports the breezy.bzr module to ensure Bazaar functionality is available.
 pub fn init_bzr() {
-    pyo3::Python::with_gil(|py| {
+    pyo3::Python::attach(|py| {
         py.import("breezy.bzr").unwrap();
     })
 }
 
 #[cfg(feature = "auto-initialize")]
+/// Initialize
 #[ctor::ctor]
 fn ensure_initialized() {
     init();
@@ -138,7 +139,10 @@ const MINIMUM_VERSION: (usize, usize, usize) = (3, 3, 6);
 /// Initialization lock to ensure Breezy is only initialized once.
 static INIT_BREEZY: Once = Once::new();
 
-/// Initialize the Breezy library.
+/// Initialize the Breezy library and Python interpreter.
+///
+/// This function ensures Python is initialized and Breezy is loaded.
+/// It is safe to call multiple times.
 ///
 /// This function ensures that Breezy is properly initialized, checking version
 /// compatibility and loading required modules. It should be called before
@@ -151,8 +155,8 @@ static INIT_BREEZY: Once = Once::new();
 /// - If the installed Breezy version is too old
 pub fn init() {
     INIT_BREEZY.call_once(|| {
-        pyo3::prepare_freethreaded_python();
-        let (major, minor, micro) = pyo3::Python::with_gil(|py| match py.import("breezy") {
+        pyo3::Python::initialize();
+        let (major, minor, micro) = pyo3::Python::attach(|py| match py.import("breezy") {
             Ok(breezy) => {
                 let (major, minor, micro, _releaselevel, _serial): (
                     usize,
@@ -187,14 +191,14 @@ pub fn init() {
         init_bzr();
 
         // Work around a breezy bug
-        pyo3::Python::with_gil(|py| {
+        pyo3::Python::attach(|py| {
             let m = py.import("breezy.controldir").unwrap();
             let f = m.getattr("ControlDirFormat").unwrap();
             f.call_method0("known_formats").unwrap();
         });
 
         // Prevent race conditions
-        pyo3::Python::with_gil(|py| {
+        pyo3::Python::attach(|py| {
             let m = py.import("breezy.config").unwrap();
             m.call_method0("GlobalStack").unwrap();
             m.call_method1("LocationStack", ("file:///",)).unwrap();
