@@ -69,8 +69,10 @@ pub struct Conflict {
     pub message: Option<String>,
 }
 
-impl FromPyObject<'_> for Conflict {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Conflict {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let path: String = ob.getattr("path")?.extract()?;
         let conflict_type: String = ob.getattr("typestring")?.extract()?;
         let message: Option<String> = ob.getattr("message").ok().and_then(|m| m.extract().ok());
@@ -173,8 +175,10 @@ impl<'py> pyo3::IntoPyObject<'py> for Kind {
     }
 }
 
-impl pyo3::FromPyObject<'_> for Kind {
-    fn extract_bound(ob: &Bound<pyo3::PyAny>) -> pyo3::PyResult<Self> {
+impl<'a, 'py> pyo3::FromPyObject<'a, 'py> for Kind {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, pyo3::PyAny>) -> PyResult<Self> {
         let s: String = ob.extract()?;
         match s.as_str() {
             "file" => Ok(Kind::File),
@@ -224,8 +228,10 @@ pub enum TreeEntry {
     },
 }
 
-impl FromPyObject<'_> for TreeEntry {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for TreeEntry {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let kind: String = ob.getattr("kind")?.extract()?;
         match kind.as_str() {
             "file" => {
@@ -504,7 +510,7 @@ pub trait Tree {
 /// This trait is implemented by all tree types that wrap Python objects.
 pub trait PyTree: Tree + std::any::Any {
     /// Get the underlying Python object for this tree.
-    fn to_object(&self, py: Python) -> PyObject;
+    fn to_object(&self, py: Python) -> Py<PyAny>;
 }
 
 impl dyn PyTree {
@@ -516,7 +522,7 @@ impl dyn PyTree {
 
 impl<T: PyTree + ?Sized> Tree for T {
     fn get_tag_dict(&self) -> Result<std::collections::HashMap<String, RevisionId>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let branch = self.to_object(py).getattr(py, "branch")?;
             let tags = branch.getattr(py, "tags")?;
             let tag_dict = tags.call_method0(py, intern!(py, "get_tag_dict"))?;
@@ -526,7 +532,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn get_file(&self, path: &Path) -> Result<Box<dyn std::io::Read>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let f = self
                 .to_object(py)
@@ -539,27 +545,27 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn get_file_text(&self, path: &Path) -> Result<Vec<u8>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let text = self
                 .to_object(py)
                 .call_method1(py, "get_file_text", (path_str,))?;
-            text.extract(py).map_err(|e| e.into())
+            text.extract(py).map_err(Into::into)
         })
     }
 
     fn get_file_lines(&self, path: &Path) -> Result<Vec<Vec<u8>>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let lines = self
                 .to_object(py)
                 .call_method1(py, "get_file_lines", (path_str,))?;
-            lines.extract(py).map_err(|e| e.into())
+            lines.extract(py).map_err(Into::into)
         })
     }
 
     fn lock_read(&self) -> Result<Lock, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let lock = self
                 .to_object(py)
                 .call_method0(py, intern!(py, "lock_read"))?;
@@ -568,7 +574,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn has_filename(&self, path: &Path) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, intern!(py, "has_filename"), (path_str,))
@@ -578,17 +584,17 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn get_symlink_target(&self, path: &Path) -> Result<PathBuf, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let target = self
                 .to_object(py)
                 .call_method1(py, "get_symlink_target", (path_str,))?;
-            target.extract(py).map_err(|e| e.into())
+            target.extract(py).map_err(Into::into)
         })
     }
 
     fn get_parent_ids(&self) -> Result<Vec<RevisionId>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             Ok(self
                 .to_object(py)
                 .call_method0(py, intern!(py, "get_parent_ids"))
@@ -598,7 +604,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn is_ignored(&self, path: &Path) -> Option<String> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "is_ignored", (path_str,))
@@ -609,18 +615,18 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn kind(&self, path: &Path) -> Result<Kind, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "kind", (path_str,))
                 .unwrap()
                 .extract(py)
-                .map_err(|e| e.into())
+                .map_err(Into::into)
         })
     }
 
     fn is_versioned(&self, path: &Path) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "is_versioned", (path_str,))
@@ -637,7 +643,7 @@ impl<T: PyTree + ?Sized> Tree for T {
         want_unversioned: Option<bool>,
         require_versioned: Option<bool>,
     ) -> Result<Box<dyn Iterator<Item = Result<TreeChange, Error>>>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let kwargs = pyo3::types::PyDict::new(py);
             if let Some(specific_files) = specific_files {
                 kwargs.set_item(
@@ -654,13 +660,13 @@ impl<T: PyTree + ?Sized> Tree for T {
             if let Some(require_versioned) = require_versioned {
                 kwargs.set_item("require_versioned", require_versioned)?;
             }
-            struct TreeChangeIter(pyo3::PyObject);
+            struct TreeChangeIter(pyo3::Py<PyAny>);
 
             impl Iterator for TreeChangeIter {
                 type Item = Result<TreeChange, Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -674,7 +680,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -691,7 +697,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn has_versioned_directories(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "has_versioned_directories")
                 .unwrap()
@@ -701,7 +707,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn preview_transform(&self) -> Result<crate::transform::TreeTransform, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let transform = self.to_object(py).call_method0(py, "preview_transform")?;
             Ok(crate::transform::TreeTransform::from(transform))
         })
@@ -715,7 +721,7 @@ impl<T: PyTree + ?Sized> Tree for T {
         recurse_nested: Option<bool>,
     ) -> Result<Box<dyn Iterator<Item = Result<(PathBuf, bool, Kind, TreeEntry), Error>>>, Error>
     {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let kwargs = pyo3::types::PyDict::new(py);
             if let Some(include_root) = include_root {
                 kwargs.set_item("include_root", include_root)?;
@@ -729,13 +735,13 @@ impl<T: PyTree + ?Sized> Tree for T {
             if let Some(recurse_nested) = recurse_nested {
                 kwargs.set_item("recurse_nested", recurse_nested)?;
             }
-            struct ListFilesIter(pyo3::PyObject);
+            struct ListFilesIter(pyo3::Py<PyAny>);
 
             impl Iterator for ListFilesIter {
                 type Item = Result<(PathBuf, bool, Kind, TreeEntry), Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -749,7 +755,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -772,14 +778,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         &self,
         path: &std::path::Path,
     ) -> Result<Box<dyn Iterator<Item = Result<(PathBuf, Kind, TreeEntry), Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct IterChildEntriesIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct IterChildEntriesIter(pyo3::Py<PyAny>);
 
             impl Iterator for IterChildEntriesIter {
                 type Item = Result<(PathBuf, Kind, TreeEntry), Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -793,7 +799,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -812,12 +818,12 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn get_file_size(&self, path: &Path) -> Result<u64, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let size = self
                 .to_object(py)
                 .call_method1(py, "get_file_size", (path_str,))?;
-            size.extract(py).map_err(|e| e.into())
+            size.extract(py).map_err(Into::into)
         })
     }
 
@@ -826,47 +832,47 @@ impl<T: PyTree + ?Sized> Tree for T {
         path: &Path,
         _stat_value: Option<&std::fs::Metadata>,
     ) -> Result<String, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let sha1 = self
                 .to_object(py)
                 .call_method1(py, "get_file_sha1", (path_str,))?;
-            sha1.extract(py).map_err(|e| e.into())
+            sha1.extract(py).map_err(Into::into)
         })
     }
 
     fn get_file_mtime(&self, path: &Path) -> Result<u64, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let mtime = self
                 .to_object(py)
                 .call_method1(py, "get_file_mtime", (path_str,))?;
-            mtime.extract(py).map_err(|e| e.into())
+            mtime.extract(py).map_err(Into::into)
         })
     }
 
     fn is_executable(&self, path: &Path) -> Result<bool, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let result = self
                 .to_object(py)
                 .call_method1(py, "is_executable", (path_str,))?;
-            result.extract(py).map_err(|e| e.into())
+            result.extract(py).map_err(Into::into)
         })
     }
 
     fn stored_kind(&self, path: &Path) -> Result<Kind, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "stored_kind", (path_str,))?
                 .extract(py)
-                .map_err(|e| e.into())
+                .map_err(Into::into)
         })
     }
 
     fn supports_content_filtering(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "supports_content_filtering")
                 .unwrap()
@@ -876,7 +882,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn supports_file_ids(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "supports_file_ids")
                 .unwrap()
@@ -886,7 +892,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn supports_rename_tracking(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "supports_rename_tracking")
                 .unwrap()
@@ -896,7 +902,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn supports_symlinks(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "supports_symlinks")
                 .unwrap()
@@ -906,7 +912,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn supports_tree_reference(&self) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "supports_tree_reference")
                 .unwrap()
@@ -916,23 +922,23 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn unknowns(&self) -> Result<Vec<PathBuf>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let unknowns = self.to_object(py).call_method0(py, "unknowns")?;
-            unknowns.extract(py).map_err(|e| e.into())
+            unknowns.extract(py).map_err(Into::into)
         })
     }
 
     fn all_versioned_paths(
         &self,
     ) -> Result<Box<dyn Iterator<Item = Result<PathBuf, Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct AllVersionedPathsIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct AllVersionedPathsIter(pyo3::Py<PyAny>);
 
             impl Iterator for AllVersionedPathsIter {
                 type Item = Result<PathBuf, Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -946,7 +952,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -960,21 +966,21 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn conflicts(&self) -> Result<Vec<Conflict>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let conflicts = self.to_object(py).call_method0(py, "conflicts")?;
-            conflicts.extract(py).map_err(|e| e.into())
+            conflicts.extract(py).map_err(Into::into)
         })
     }
 
     fn extras(&self) -> Result<Vec<PathBuf>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let extras = self.to_object(py).call_method0(py, "extras")?;
-            extras.extract(py).map_err(|e| e.into())
+            extras.extract(py).map_err(Into::into)
         })
     }
 
     fn filter_unversioned_files(&self, paths: &[&Path]) -> Result<Vec<PathBuf>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_strings: Vec<String> = paths
                 .iter()
                 .map(|p| p.to_string_lossy().to_string())
@@ -982,7 +988,7 @@ impl<T: PyTree + ?Sized> Tree for T {
             let result =
                 self.to_object(py)
                     .call_method1(py, "filter_unversioned_files", (path_strings,))?;
-            result.extract(py).map_err(|e| e.into())
+            result.extract(py).map_err(Into::into)
         })
     }
 
@@ -990,14 +996,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         &self,
         prefix: Option<&Path>,
     ) -> Result<Box<dyn Iterator<Item = Result<WalkdirResult, Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct WalkdirsIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct WalkdirsIter(pyo3::Py<PyAny>);
 
             impl Iterator for WalkdirsIter {
                 type Item = Result<WalkdirResult, Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1012,7 +1018,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                             None
                         } else {
                             let tuple = match next
-                                .extract::<(String, String, String, Option<PyObject>, bool)>(py)
+                                .extract::<(String, String, String, Option<Py<PyAny>>, bool)>(py)
                             {
                                 Ok(t) => t,
                                 Err(e) => return Some(Err(e.into())),
@@ -1041,7 +1047,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn versionable_kind(&self, kind: &Kind) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method1(py, "versionable_kind", (kind.clone(),))
                 .unwrap()
@@ -1051,7 +1057,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn path_content_summary(&self, path: &Path) -> Result<PathContentSummary, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let summary =
                 self.to_object(py)
@@ -1090,14 +1096,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         &self,
         paths: &[&Path],
     ) -> Result<Box<dyn Iterator<Item = Result<(PathBuf, Vec<u8>), Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct IterFilesBytesIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct IterFilesBytesIter(pyo3::Py<PyAny>);
 
             impl Iterator for IterFilesBytesIter {
                 type Item = Result<(PathBuf, Vec<u8>), Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1111,7 +1117,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -1136,14 +1142,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         &self,
         specific_files: Option<&[&Path]>,
     ) -> Result<Box<dyn Iterator<Item = Result<(PathBuf, TreeEntry), Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct IterEntriesByDirIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct IterEntriesByDirIter(pyo3::Py<PyAny>);
 
             impl Iterator for IterEntriesByDirIter {
                 type Item = Result<(PathBuf, TreeEntry), Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1157,7 +1163,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -1189,22 +1195,22 @@ impl<T: PyTree + ?Sized> Tree for T {
         path: &Path,
         _stat_value: Option<&std::fs::Metadata>,
     ) -> Result<(String, Vec<u8>), Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let result = self
                 .to_object(py)
                 .call_method1(py, "get_file_verifier", (path_str,))?;
-            result.extract(py).map_err(|e| e.into())
+            result.extract(py).map_err(Into::into)
         })
     }
 
     fn get_reference_revision(&self, path: &Path) -> Result<RevisionId, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             let rev = self
                 .to_object(py)
                 .call_method1(py, "get_reference_revision", (path_str,))?;
-            rev.extract(py).map_err(|e| e.into())
+            rev.extract(py).map_err(Into::into)
         })
     }
 
@@ -1217,14 +1223,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         force_mtime: Option<f64>,
         recurse_nested: bool,
     ) -> Result<Box<dyn Iterator<Item = Result<Vec<u8>, Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct ArchiveIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct ArchiveIter(pyo3::Py<PyAny>);
 
             impl Iterator for ArchiveIter {
                 type Item = Result<Vec<u8>, Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1238,7 +1244,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -1273,14 +1279,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         path: &Path,
         default_revision: Option<&RevisionId>,
     ) -> Result<Box<dyn Iterator<Item = Result<(RevisionId, Vec<u8>), Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct AnnotateIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct AnnotateIter(pyo3::Py<PyAny>);
 
             impl Iterator for AnnotateIter {
                 type Item = Result<(RevisionId, Vec<u8>), Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1294,7 +1300,7 @@ impl<T: PyTree + ?Sized> Tree for T {
                         if next.is_none(py) {
                             None
                         } else {
-                            Some(next.extract(py).map_err(|e| e.into()))
+                            Some(next.extract(py).map_err(Into::into))
                         }
                     })
                 }
@@ -1322,7 +1328,7 @@ impl<T: PyTree + ?Sized> Tree for T {
     }
 
     fn is_special_path(&self, path: &Path) -> bool {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "is_special_path", (path_str,))
@@ -1336,14 +1342,14 @@ impl<T: PyTree + ?Sized> Tree for T {
         &self,
         paths: &[&Path],
     ) -> Result<Box<dyn Iterator<Item = Result<SearchRule, Error>>>, Error> {
-        Python::with_gil(|py| {
-            struct IterSearchRulesIter(pyo3::PyObject);
+        Python::attach(|py| {
+            struct IterSearchRulesIter(pyo3::Py<PyAny>);
 
             impl Iterator for IterSearchRulesIter {
                 type Item = Result<SearchRule, Error>;
 
                 fn next(&mut self) -> Option<Self::Item> {
-                    Python::with_gil(|py| {
+                    Python::attach(|py| {
                         let next = match self.0.call_method0(py, intern!(py, "__next__")) {
                             Ok(v) => v,
                             Err(e) => {
@@ -1400,7 +1406,7 @@ impl<T: PyTree + ?Sized> Tree for T {
 }
 
 /// A generic tree implementation that wraps any Python tree object.
-pub struct GenericTree(PyObject);
+pub struct GenericTree(Py<PyAny>);
 
 impl<'py> IntoPyObject<'py> for GenericTree {
     type Target = PyAny;
@@ -1412,14 +1418,14 @@ impl<'py> IntoPyObject<'py> for GenericTree {
     }
 }
 
-impl From<PyObject> for GenericTree {
-    fn from(obj: PyObject) -> Self {
+impl From<Py<PyAny>> for GenericTree {
+    fn from(obj: Py<PyAny>) -> Self {
         GenericTree(obj)
     }
 }
 
 impl PyTree for GenericTree {
-    fn to_object(&self, py: Python) -> PyObject {
+    fn to_object(&self, py: Python) -> Py<PyAny> {
         self.0.clone_ref(py)
     }
 }
@@ -1486,7 +1492,7 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
         for f in files {
             assert!(f.is_relative());
         }
-        Python::with_gil(|py| -> Result<(), PyErr> {
+        Python::attach(|py| -> Result<(), PyErr> {
             let path_strings: Vec<String> = files
                 .iter()
                 .map(|p| p.to_string_lossy().to_string())
@@ -1495,11 +1501,11 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
                 .call_method1(py, "add", (path_strings,))?;
             Ok(())
         })
-        .map_err(|e| e.into())
+        .map_err(Into::into)
     }
 
     fn lock_write(&self) -> Result<Lock, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let lock = self
                 .to_object(py)
                 .call_method0(py, intern!(py, "lock_write"))?;
@@ -1509,7 +1515,7 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
 
     fn put_file_bytes_non_atomic(&self, path: &Path, data: &[u8]) -> Result<(), Error> {
         assert!(path.is_relative());
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py)
                 .call_method1(py, "put_file_bytes_non_atomic", (path_str, data))?;
@@ -1518,29 +1524,29 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     }
 
     fn has_changes(&self) -> std::result::Result<bool, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "has_changes")?
                 .extract::<bool>(py)
-                .map_err(|e| e.into())
+                .map_err(Into::into)
         })
     }
 
     fn mkdir(&self, path: &Path) -> Result<(), Error> {
         assert!(path.is_relative());
-        Python::with_gil(|py| -> Result<(), PyErr> {
+        Python::attach(|py| -> Result<(), PyErr> {
             let path_str = path.to_string_lossy().to_string();
             self.to_object(py).call_method1(py, "mkdir", (path_str,))?;
             Ok(())
         })
-        .map_err(|e| e.into())
+        .map_err(Into::into)
     }
 
     fn remove(&self, files: &[&std::path::Path]) -> Result<(), Error> {
         for f in files {
             assert!(f.is_relative());
         }
-        Python::with_gil(|py| -> Result<(), PyErr> {
+        Python::attach(|py| -> Result<(), PyErr> {
             let path_strings: Vec<String> = files
                 .iter()
                 .map(|p| p.to_string_lossy().to_string())
@@ -1549,11 +1555,11 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
                 .call_method1(py, "remove", (path_strings,))?;
             Ok(())
         })
-        .map_err(|e| e.into())
+        .map_err(Into::into)
     }
 
     fn add_reference(&self, reference: &TreeReference) -> Result<(), Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let kwargs = pyo3::types::PyDict::new(py);
             kwargs.set_item("path", reference.path.to_string_lossy().to_string())?;
             kwargs.set_item("kind", reference.kind.clone())?;
@@ -1569,7 +1575,7 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     fn copy_one(&self, from_path: &Path, to_path: &Path) -> Result<(), Error> {
         assert!(from_path.is_relative());
         assert!(to_path.is_relative());
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let from_str = from_path.to_string_lossy().to_string();
             let to_str = to_path.to_string_lossy().to_string();
             self.to_object(py)
@@ -1579,7 +1585,7 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     }
 
     fn last_revision(&self) -> Result<RevisionId, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let last_revision = self
                 .to_object(py)
                 .call_method0(py, intern!(py, "last_revision"))?;
@@ -1588,15 +1594,15 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     }
 
     fn lock_tree_write(&self) -> Result<Lock, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let lock = self.to_object(py).call_method0(py, "lock_tree_write")?;
             Ok(Lock::from(lock))
         })
     }
 
     fn set_parent_ids(&self, parent_ids: &[RevisionId]) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            let parent_ids_py: Vec<PyObject> = parent_ids
+        Python::attach(|py| {
+            let parent_ids_py: Vec<Py<PyAny>> = parent_ids
                 .iter()
                 .map(|id| id.clone().into_pyobject(py).unwrap().unbind())
                 .collect();
@@ -1607,8 +1613,8 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     }
 
     fn set_parent_trees(&self, parent_trees: &[(RevisionId, RevisionTree)]) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            let parent_trees_py: Vec<(PyObject, PyObject)> = parent_trees
+        Python::attach(|py| {
+            let parent_trees_py: Vec<(Py<PyAny>, Py<PyAny>)> = parent_trees
                 .iter()
                 .map(|(id, tree)| {
                     (
@@ -1624,8 +1630,8 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
     }
 
     fn apply_inventory_delta(&self, delta: Vec<InventoryDelta>) -> Result<(), Error> {
-        Python::with_gil(|py| {
-            let delta_py: Vec<PyObject> = delta
+        Python::attach(|py| {
+            let delta_py: Vec<Py<PyAny>> = delta
                 .into_iter()
                 .map(|d| {
                     let tuple = pyo3::types::PyTuple::new(
@@ -1663,7 +1669,7 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
         allow_pointless: Option<bool>,
         specific_files: Option<&[&Path]>,
     ) -> Result<RevisionId, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let kwargs = pyo3::types::PyDict::new(py);
             if let Some(committer) = committer {
                 kwargs.set_item("committer", committer)?;
@@ -1684,13 +1690,13 @@ impl<T: PyMutableTree + ?Sized> MutableTree for T {
             let result = self
                 .to_object(py)
                 .call_method(py, "commit", (message,), Some(&kwargs))?;
-            result.extract(py).map_err(|e| e.into())
+            result.extract(py).map_err(Into::into)
         })
     }
 }
 
 /// A read-only tree at a specific revision.
-pub struct RevisionTree(pub PyObject);
+pub struct RevisionTree(pub Py<PyAny>);
 
 impl<'py> IntoPyObject<'py> for RevisionTree {
     type Target = PyAny;
@@ -1703,21 +1709,21 @@ impl<'py> IntoPyObject<'py> for RevisionTree {
 }
 
 impl PyTree for RevisionTree {
-    fn to_object(&self, py: Python) -> PyObject {
+    fn to_object(&self, py: Python) -> Py<PyAny> {
         self.0.clone_ref(py)
     }
 }
 
 impl Clone for RevisionTree {
     fn clone(&self) -> Self {
-        Python::with_gil(|py| RevisionTree(self.0.clone_ref(py)))
+        Python::attach(|py| RevisionTree(self.0.clone_ref(py)))
     }
 }
 
 impl RevisionTree {
     /// Get the repository this revision tree belongs to.
     pub fn repository(&self) -> crate::repository::GenericRepository {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let repository = self.to_object(py).getattr(py, "_repository").unwrap();
             crate::repository::GenericRepository::new(repository)
         })
@@ -1725,7 +1731,7 @@ impl RevisionTree {
 
     /// Get the revision ID of this tree.
     pub fn get_revision_id(&self) -> RevisionId {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, "get_revision_id")
                 .unwrap()
@@ -1736,7 +1742,7 @@ impl RevisionTree {
 
     /// Get the parent revision IDs of this tree.
     pub fn get_parent_ids(&self) -> Vec<RevisionId> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method0(py, intern!(py, "get_parent_ids"))
                 .unwrap()
@@ -1797,8 +1803,10 @@ impl<'py> IntoPyObject<'py> for TreeChange {
     }
 }
 
-impl FromPyObject<'_> for TreeChange {
-    fn extract_bound(obj: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for TreeChange {
+    type Error = PyErr;
+
+    fn extract(obj: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         fn from_bool(o: &Bound<PyAny>) -> PyResult<bool> {
             if let Ok(b) = o.extract::<isize>() {
                 Ok(b != 0)
@@ -1837,7 +1845,7 @@ impl FromPyObject<'_> for TreeChange {
 }
 
 /// An in-memory tree implementation.
-pub struct MemoryTree(pub PyObject);
+pub struct MemoryTree(pub Py<PyAny>);
 
 impl<'py> IntoPyObject<'py> for MemoryTree {
     type Target = PyAny;
@@ -1851,7 +1859,7 @@ impl<'py> IntoPyObject<'py> for MemoryTree {
 
 impl<B: crate::branch::PyBranch> From<&B> for MemoryTree {
     fn from(branch: &B) -> Self {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             MemoryTree(
                 branch
                     .to_object(py)
@@ -1865,7 +1873,7 @@ impl<B: crate::branch::PyBranch> From<&B> for MemoryTree {
 }
 
 impl PyTree for MemoryTree {
-    fn to_object(&self, py: Python) -> PyObject {
+    fn to_object(&self, py: Python) -> Py<PyAny> {
         self.0.clone_ref(py)
     }
 }
