@@ -145,7 +145,7 @@ pub trait VersionedFiles: Clone + Send + Sync {
 }
 
 pub trait PyVersionedFiles: VersionedFiles {
-    fn to_object(&self, py: Python) -> PyObject;
+    fn to_object(&self, py: Python) -> Py<PyAny>;
 }
 
 impl<T: PyVersionedFiles> VersionedFiles for T {
@@ -160,7 +160,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
         random_id: bool,
         check_content: bool,
     ) -> Result<(String, usize), Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let parents_py: Vec<_> = parents
                 .iter()
                 .map(|p| p.clone().into_pyobject(py))
@@ -200,11 +200,11 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
             )?;
 
             let tuple = result
-                .downcast_bound::<PyTuple>(py)
+                .cast_bound::<PyTuple>(py)
                 .map_err(|_| PyValueError::new_err("Expected tuple"))?;
             let item0 = tuple.get_item(0)?;
             let sha1_bytes = item0
-                .downcast::<PyBytes>()
+                .cast::<PyBytes>()
                 .map_err(|_| PyValueError::new_err("Expected bytes"))?;
             let sha1 = std::str::from_utf8(sha1_bytes.as_bytes())
                 .map_err(|_| PyValueError::new_err("Invalid UTF-8 in SHA1"))?
@@ -221,7 +221,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
         ordering: RecordOrdering,
         include_delta_closure: bool,
     ) -> Result<RecordStream, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_py: Vec<_> = keys
                 .into_iter()
                 .map(|k| k.into_pyobject(py))
@@ -245,7 +245,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn get_sha1s(&self, keys: Vec<Key>) -> Result<HashMap<Key, String>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_py: Vec<_> = keys
                 .into_iter()
                 .map(|k| k.into_pyobject(py))
@@ -257,14 +257,14 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
                 .call_method1(py, "get_sha1s", (keys_list,))?;
 
             let dict = result_dict
-                .downcast_bound::<PyDict>(py)
+                .cast_bound::<PyDict>(py)
                 .map_err(|_| PyValueError::new_err("Expected dict"))?;
             let mut sha1s = HashMap::new();
 
             for (key_py, sha_py) in dict {
-                let key = Key::extract_bound(&key_py)?;
+                let key = key_py.extract::<Key>()?;
                 let sha_bytes = sha_py
-                    .downcast::<PyBytes>()
+                    .cast::<PyBytes>()
                     .map_err(|_| PyValueError::new_err("Expected bytes"))?;
                 let sha = std::str::from_utf8(sha_bytes.as_bytes())
                     .map_err(|_| PyValueError::new_err("Invalid UTF-8 in SHA1"))?
@@ -277,7 +277,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn insert_record_stream(&self, stream: RecordStream) -> Result<(), Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             self.to_object(py)
                 .call_method1(py, "insert_record_stream", (stream.0,))?;
             Ok(())
@@ -285,15 +285,15 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn keys(&self) -> Result<Vec<Key>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_iter = self.to_object(py).call_method0(py, "keys")?;
 
             let mut keys = Vec::new();
             for key_py in keys_iter
-                .downcast_bound::<PyIterator>(py)
+                .cast_bound::<PyIterator>(py)
                 .map_err(|_| PyValueError::new_err("Expected iterator"))?
             {
-                let key = Key::extract_bound(&key_py?)?;
+                let key = key_py?.extract::<Key>()?;
                 keys.push(key);
             }
 
@@ -302,7 +302,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn make_mpdiffs(&self, keys: Vec<Key>) -> Result<Vec<MultiParentDiff>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_py: Vec<_> = keys
                 .into_iter()
                 .map(|k| k.into_pyobject(py))
@@ -315,10 +315,10 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
 
             let mut diffs = Vec::new();
             for diff_py in result
-                .downcast_bound::<PyIterator>(py)
+                .cast_bound::<PyIterator>(py)
                 .map_err(|_| PyValueError::new_err("Expected iterator"))?
             {
-                let diff = MultiParentDiff::extract_bound(&diff_py?)?;
+                let diff = diff_py?.extract::<MultiParentDiff>()?;
                 diffs.push(diff);
             }
 
@@ -327,7 +327,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn get_parent_map(&self, keys: Vec<Key>) -> Result<HashMap<Key, Vec<Key>>, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_py: Vec<_> = keys
                 .into_iter()
                 .map(|k| k.into_pyobject(py))
@@ -339,18 +339,18 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
                     .call_method1(py, "get_parent_map", (keys_list,))?;
 
             let dict = result_dict
-                .downcast_bound::<PyDict>(py)
+                .cast_bound::<PyDict>(py)
                 .map_err(|_| PyValueError::new_err("Expected dict"))?;
             let mut parent_map = HashMap::new();
 
             for (key_py, parents_py) in dict {
-                let key = Key::extract_bound(&key_py)?;
+                let key = key_py.extract::<Key>()?;
                 let mut parents = Vec::new();
                 for parent_py in parents_py
-                    .downcast::<PyTuple>()
+                    .cast::<PyTuple>()
                     .map_err(|_| PyValueError::new_err("Expected tuple"))?
                 {
-                    let parent = Key::extract_bound(&parent_py)?;
+                    let parent = parent_py.extract::<Key>()?;
                     parents.push(parent);
                 }
                 parent_map.insert(key, parents);
@@ -361,7 +361,7 @@ impl<T: PyVersionedFiles> VersionedFiles for T {
     }
 
     fn get_known_graph_ancestry(&self, keys: Vec<Key>) -> Result<KnownGraph, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let keys_py: Vec<_> = keys
                 .into_iter()
                 .map(|k| k.into_pyobject(py))
@@ -384,25 +384,25 @@ pub enum RecordOrdering {
     GroupedByKey,
 }
 
-pub struct RecordStream(PyObject);
+pub struct RecordStream(Py<PyAny>);
 
 impl RecordStream {
     pub fn iter(&self) -> Result<RecordStreamIterator, Error> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let iter = self.0.call_method0(py, "__iter__")?;
             Ok(RecordStreamIterator(iter))
         })
     }
 }
 
-pub struct RecordStreamIterator(PyObject);
+pub struct RecordStreamIterator(Py<PyAny>);
 
 impl Iterator for RecordStreamIterator {
     type Item = Result<Record, Error>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        Python::with_gil(|py| match self.0.call_method0(py, "__next__") {
-            Ok(record_py) => Some(Record::extract_bound(&record_py.bind(py)).map_err(Into::into)),
+        Python::attach(|py| match self.0.call_method0(py, "__next__") {
+            Ok(record_py) => Some(record_py.bind(py).extract::<Record>().map_err(Into::into)),
             Err(e) if e.is_instance_of::<pyo3::exceptions::PyStopIteration>(py) => None,
             Err(e) => Some(Err(e.into())),
         })
@@ -417,15 +417,17 @@ pub struct Record {
     pub parents: Vec<Key>,
 }
 
-impl<'py> FromPyObject<'py> for Record {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Record {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let key = ob.getattr("key")?.extract::<Key>()?;
         let storage_kind = ob.getattr("storage_kind")?.extract::<String>()?;
 
         let sha1 = if let Ok(sha_bytes) = ob.getattr("sha1") {
             if !sha_bytes.is_none() {
                 let bytes = sha_bytes
-                    .downcast::<PyBytes>()
+                    .cast::<PyBytes>()
                     .map_err(|_| PyValueError::new_err("Expected bytes"))?;
                 Some(
                     std::str::from_utf8(bytes.as_bytes())
@@ -457,10 +459,12 @@ pub struct MultiParentDiff {
     pub hunks: Vec<DiffHunk>,
 }
 
-impl<'py> FromPyObject<'py> for MultiParentDiff {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for MultiParentDiff {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let tuple = ob
-            .downcast::<PyTuple>()
+            .cast::<PyTuple>()
             .map_err(|_| PyValueError::new_err("Expected tuple"))?;
         let key = tuple.get_item(0)?.extract::<Key>()?;
         let parents = tuple.get_item(1)?.extract::<Vec<Key>>()?;
@@ -468,10 +472,10 @@ impl<'py> FromPyObject<'py> for MultiParentDiff {
         let hunks_py = tuple.get_item(2)?;
         let mut hunks = Vec::new();
         for hunk_py in hunks_py
-            .downcast::<PyList>()
+            .cast::<PyList>()
             .map_err(|_| PyValueError::new_err("Expected list"))?
         {
-            hunks.push(DiffHunk::extract_bound(&hunk_py)?);
+            hunks.push(hunk_py.extract::<DiffHunk>()?);
         }
 
         Ok(MultiParentDiff {
@@ -492,10 +496,12 @@ pub enum DiffHunk {
     },
 }
 
-impl<'py> FromPyObject<'py> for DiffHunk {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for DiffHunk {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let tuple = ob
-            .downcast::<PyTuple>()
+            .cast::<PyTuple>()
             .map_err(|_| PyValueError::new_err("Expected tuple"))?;
         let hunk_type = tuple.get_item(0)?.extract::<String>()?;
 
@@ -518,22 +524,22 @@ impl<'py> FromPyObject<'py> for DiffHunk {
     }
 }
 
-pub struct GenericVersionedFiles(PyObject);
+pub struct GenericVersionedFiles(Py<PyAny>);
 
 impl GenericVersionedFiles {
-    pub fn new(py_obj: PyObject) -> Self {
+    pub fn new(py_obj: Py<PyAny>) -> Self {
         Self(py_obj)
     }
 }
 
 impl Clone for GenericVersionedFiles {
     fn clone(&self) -> Self {
-        Python::with_gil(|py| GenericVersionedFiles(self.0.clone_ref(py)))
+        Python::attach(|py| GenericVersionedFiles(self.0.clone_ref(py)))
     }
 }
 
 impl PyVersionedFiles for GenericVersionedFiles {
-    fn to_object(&self, py: Python) -> PyObject {
+    fn to_object(&self, py: Python) -> Py<PyAny> {
         self.0.clone_ref(py)
     }
 }
@@ -548,9 +554,11 @@ impl<'py> IntoPyObject<'py> for GenericVersionedFiles {
     }
 }
 
-impl<'py> FromPyObject<'py> for GenericVersionedFiles {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
-        Ok(GenericVersionedFiles(ob.clone().unbind()))
+impl<'a, 'py> FromPyObject<'a, 'py> for GenericVersionedFiles {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        Ok(GenericVersionedFiles(ob.to_owned().unbind()))
     }
 }
 
